//******************************************************************************
//
//
//    KEY FINDER (PROGRAMVEL)
//
//
//    Autor: Claudio Larios
//
//    Data: 21/11/2017
//
//    Dispositivo Eletrnico para localizao de mlho de chaves.
//    Funciona com um assovio, aps o que emite um bip-bip.
//    Permite programar a sua opo de assovio.
//
//==============================================================================

#include <12F675.h>
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOPUT, NOPROTECT, NOBROWNOUT, NOMCLR,  NOCPD
#ROM 0X2100 = {42}// Frequncia do assovio de 950 Hz a 1050Hz

#byte  tmr0      = 0x01
#byte  gpio      = 0x05
#byte  tris      = 0x85
#byte  intcon    = 0x0b
#bit   sinal     = 0x5.2
#bit   sw_prog   = 0x5.3
#bit   t0if      = 0x0b.2  //flag over flow do tmr0

//variveis globais
unsigned int1 f_time=0;
unsigned int8  cnt_sinal=0;


/*******************************************************************************
          ROTINA DE TESTE IDENTIFICADORA DE UM ASSOVIO
*******************************************************************************/
void testa_sinal(){
  unsigned int8 valor,eep,n;
  //define a janela de frequncia desejada para o assovio, tempo de permanncia
  // mnima do assovio e frequncia mnima.
  //Nota: A frequncia do assovio tem de estar entre 500 a 5000 Hz.

  #define  TOL    5    //Tolerncia no assovio (Janela.Freq= TOL * 20 Hz).
                       // uma janela de frequncia que ser detectada.
                       // Pode ser estreita (ex.1=20Hz) ou larga (ex.10=200Hz).
                       //Valores baixos exigem preciso na frequncia do assovio.
  #define  PER    10    //Nvel de permanncia do assovio na entrada (filtragem).
                       // Valores maiores necessitam mais tempo assoviando
                       // para identificao ser bem sucedida.
  #define  FREQ_IN_MIN  25 //frequncia mnima do assovio (25 * 20= 500Hz)
                       //Limita a frequncia mnima para evitar que sons graves
                       // de som de carros acionem o dispositivo erradicamente.
  
    n=PER;//nvel de filtragem para som ambiente
    eep=read_eeprom(0);//le frequncia gravada na EEprom
    
    do{
      f_time=0;//reseta flag de habilitao de bips
      set_timer0(0);//zera timer 0
      t0if=0;//reseta flag de over-flow do timer 0
      delay_ms(50);//delay de 50 useg
      valor=tmr0;//salva leitura do timer 0
      if(t0if)return;//se houve estouro - erro (desconsidera valores obtidos)

      if(!sw_prog){//se foi apertado boto de programao, grava assovio
      if(valor<FREQ_IN_MIN)return;//se freq menor que a mnima - erro 
       write_eeprom(0,valor);//salva frequncia do assovio
       gpio=0b100000;//liga led para indicar programao bem sucedida
       while(!sw_prog);delay_ms(500);//aguarda solta boto de programao
       gpio=0;//apaga led
       return;//sai da rotina
    }
      if((valor+TOL>=eep)&&(valor<=(eep+TOL)))n--; else return;//compara freq.
      delay_us(25);//delay entre capturas
    }while(n);//captura por n vezes (filtro)
    
    f_time=1;//assovio ok, autoriza produo de bips no disco piezo eltrico
}

/*******************************************************************************
          ROTINA DE GERAO DE BIPS
*******************************************************************************/
void avisa(){
   unsigned int8 a,b;

   if(!f_time) return; //sinal incorreto
   delay_ms(500);//delay para reproduzir beep aps o apito

   for(b=0;b<5;b++){//reproduz bip em tom A
     for(a=0;a<40;a++){
      gpio=0b100000;
      delay_us(1275);//392h=1275;440h=1136;1000h=500; 2900h=172; 2637h=189;2794h=179
      gpio=0b010000;
      delay_us(1275);
     }
     delay_ms(20);//intervalo entre um bip e outro
      for(a=0;a<40;a++){//reproduz bip em tom B
      gpio=0b100000;
      delay_us(1136);//282
      gpio=0b010000;
      delay_us(1136);
     }
     delay_ms(20);//intervalo entre um bip e outro
  }
  cnt_sinal=0;//zera

}

/*******************************************************************************
                   ROTINA MAIN
*******************************************************************************/
void main() {
   setup_adc_ports(no_analogs);  //somente pic12f675
   SETUP_ADC(ADC_OFF); //DESLIGA ADC
   setup_counters(RTCC_EXT_L_TO_H,RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   tris= 0b001111; //tudo sada
   gpio=0;

/*******************************************************************************
                  ROTINA PRINCIPAL
*******************************************************************************/

   for(;;){
      testa_sinal();//Verifica se houve um assovio
      avisa();//se sim, reproduz bip-bip de alerta
      delay_ms(50);//delay
   }//for(;;)
}//main()
/*******************************************************************************
                    FIM DO PROGRAMA
*******************************************************************************/
